# GTC Python Visualization Package

## Overview

This package is dedicated to provide a unified interface for GTC routine diagnostics.

The package is currently under construction and is not yet completed. New GTC programmer should use their best judgement to accept or reject the existing structure and/or codes. 

None of the features are fully tested or functional. This package is provided more like a guide for future GTC developers.

## Structure

The package consists of two major parts: the "loader" and the "visualizer".

The loader classes convert various gtc output file formats (".out", ".nc" or ".bp") into a unified python object, with predefined attributes containing relevant data.

The visualizer classes then use the loader objects to create the routine diagnostic plots.

## Loader Classes

Each Loader Class should receive a file name as input, and automatically detects the file type. It is user's responsibility to provide the correct file name and extension to the corresponding loader class. Unexpected Error will occur when a wrong file and/or wrong extension is provided, sometimes the error will occur in a later stage, for example, in visualization step.

Currently, only two loader classes are implemented. Snapshot and Equilibrium. Both are only implemented for the ADIOS2 BP file format. An older loader for history.out file exists, but needs to be updated/replaced by a newer one to obey the naming convention.

None of the ADIOS2 loaders are thoroughly tested, they are provided more like a guide for future GTC developers. 

## Visualizer Classes

Each Visualizer object maintains one matplotlib figure. The plotting methods will renew the figure with new plots. This is to mimic the existing IDL plotting behavior.

An example of implementation can be found in the history visualizer class. Note that this class is _NOT_ up to date with the overall naming conventions.

Another example is the Equilibrium visualizer class, which implements several typical types of plots. However, this class is not tested and may contain bugs.

## Future Work

Future GTC developers should follow the 3-step process to complete this package:

1. define naming convention: 
    write a list of names and their physical meanings. Implement the data writing part inside GTC main code using your naming convention.

2. implement the loader class:
    read your output data file and construct the loader object according to your naming convention

3. implement the visualizer class:
    create the visualizer class, make plots with the data provided by the loader, and customize your plots
